; Infrared PrintScreen TSR rev 1.03
; Copyright 1991 Hewlett-Packard Company
; All rights reserved
;
; This source code is provided on an as-is basis, as an example of how one
; MIGHT print to the HP 82240A "Redeye" printer.  No warantees or guarantees
; given or implied as to the functionality, practicality, usability, or any
; other 'ities' of this code.
;
; Permission is hereby given for reasonable portions of this code to be
; incorporated (modified or not) in other programs.
;

;This program prints the screen to a redeye printer
;It takes over INT5 and runs as a TSR when the print screen
;key is pressed

cseg	segment para public 'code'
	assume	cs:cseg, ds:cseg
	org	100h
start:
	jmp	init	

old1coff	dw	?
old1cseg	dw	?

buffer	db	384 dup(?)

char1	db	8 dup(?)
char2	db	8 dup(?)
char3	db	8 dup(?)
char4 	db	8 dup(?)

byt1	db	1 (?)
byt2	db	1 (?)
byt3	db	1 (?)
byt4	db	1 (?)

isr5:		
	push	ds			;save regs
	push 	es
	push 	ax
	push	bx
	push	cx
	push	dx
	push	di
	push	si

	mov	ax,50h			;check to see if in print screen
	mov	ds,ax
	cmp	byte ptr ds:[0],1
	jnz	nexit
	jmp	exit
nexit:	mov	byte ptr ds:[0],1	;set in prt screen flag
	mov	ax,cs			;point ds and es to same segment
	mov	es,ax
	mov	ds,ax
	
	mov	al,1			;redeye mode
	mov	dx,0e30ah		;irfmat register
	out	dx,al

	mov	ah,0fh			;check if in text or graphics mode
	int	10h

	cmp	al,7			;text mode
	jz	text
	jmp	graph			;graphics mode

text:
	mov	dx,0d300h		;get offset of jag display into window
	in	ax,dx
	mov	si,ax			;put offset in si
	add	si,72			;go to end of line
	lea	di,buffer		;address of current location in buffer
	mov	cx,10			;40 characters/4 at a time
	
main:	
	push	cx			;save cx
	mov	cx,16			;number of lines to read

second:
	push	di			;save di
	push	ds
	mov	ax,0b000h		;display buffer
	mov	ds,ax
	lodsw				;get character
	lea	di,char1
	call	getfont
	lodsw
	lea	di,char2
	call	getfont
	lodsw
	lea	di,char3
	call	getfont
	lodsw
	lea	di,char4
	call	getfont
	pop	ds
	pop	di

	push	cx
	push	si			;save si and cx
	mov	cx,8			;loop count
	lea	si,char1		;character offset
compress:
	lodsb
	mov	byt1,al
	add	si,7			;next char
	lodsb
	mov	byt2,al
	add	si,7			;next char
	lodsb
	mov	byt3,al
	add	si,7			;next char
	lodsb
	mov	byt4,al
	sub	si,24			;prepare for next line
	
	push	cx			;save cx
	mov	al,byt1			;compress 4 6bit words in byt1-4 to
					;3 8bit words in byt1-3
	mov	cl,2			;shift 2
	shl	al,cl
	mov	byt1,al	
	mov	al,byt2	
	xor	ah,ah			;clear ah
	mov	cl,4			;shift 4
	shl	ax,cl
	or	ah,byt1
	mov	byt1,ah
	mov	byt2,al
	mov	al,byt3
	xor	ah,ah			;clear	ah
	mov	cl,6			;shift 6
	shl	ax,cl
	or	ah,byt2
	mov	byt2,ah
	or	al,byt4
	mov	byt3,al
	pop	cx			;restore cx

	mov	al,byt1
	stosb
	mov	al,byt2
	stosb
	mov	al,byt3
	stosb
	loop	compress
	
	pop	si
	add	si,152			;prepare for next 3 characters
	pop	cx			;restore si and cx
	
	loop	jsecond
	
	call	pbuffer			;print the buffer
	
	lea	di,buffer		;prepare for next line of characters
	sub	si,2568

	pop	cx			;restore cx
	loop	jmain

	jmp	exxit			;done
jmain:	jmp	main
jsecond:jmp	second

graph:
	mov	si,0b000h
	mov	ds,si			;graphics display
	xor	si,si
	add	si,29			;go to top right of buffer 
	mov	cx,30			;line count
pline:
	mov	al,27			;esc
	call	redeye
	mov	al,128			;graphics char count
	call	redeye
	
	push	cx
	mov	cx,128			;output 128 graphics characters
lloop:
	lodsb
	call	redeye
	add	si,29			;move down one pixel line
	loop	lloop
	pop	cx

	mov	al,4			;end of line
	call	redeye
	call	sdelay

	sub	si,3841			;prepare for next line
	loop	pline
	
exxit:
	mov	al,0
	mov	dx,0e30ah
	out	dx,al

	mov	ax,50h			;set flag that we exitted ok
	mov	ds,ax
	mov	byte ptr ds:[0],0
	jmp	exit

pbuffer:
	push	si			;save si
	lea	si,buffer
	add	si,2			;go to top right of buffer 
	mov	cx,3			;line count
p1line:
	mov	al,27			;esc
	call	redeye
	mov	al,128			;graphics char count
	call	redeye
	
	push	cx
	mov	cx,128			;output 128 graphics characters
l1loop:
	lodsb
	call	redeye
	add	si,2			;move down one pixel line
	loop	l1loop
	pop	cx

	mov	al,4			;end of line
	call	redeye
	call	sdelay

	sub	si,385			;prepare for next line

	loop	p1line
	pop	si
	ret


;Gets the 8 byte font for the character in AX and stores
;them at location pointed to by di
getfont:
	push	bx
	push	si
	push	cx
	push	dx
	push	ds
	mov	cx,0f000h
	mov	ds,cx
	mov	si,0fa6eh		;1st page of font table
	cmp	al,80h			;check to see if it is 1st 128
	jb	f128
	xor	si,si
	mov	ds,si
	lds	si,ds:[1fh*4]
	sub	al,80h			;clear high bit
f128:	xor	bh,bh			;clear high byte
	mov	bl,al			;get char code
	mov	cl,3			;multiply be 8
	shl	bx,cl
	mov	cx,7			;loop count
	cmp	ah,00h			;check for blanked
	jnz	gfnxt
	mov	al,0
gfoop:	stosb				;store 0's
	loop	gfoop
	jmp	gflb2
gfnxt:
	cmp	ah,70h			;inverse video
	jz	invid
	mov	dl,0
	jmp	gflp
invid:	mov	dl,03fh			;inverse
gflp:	mov	al,ds:[bx][si]		;get byte
	xor	al,dl			;for inverse video 
	stosb
	inc	bx
	loop	gflp
	mov	al,ds:[bx][si]		;get byte
	xor	al,dl			;for inverse video
	cmp	ah,01h			;check for underlined
	jnz	gflb2
	mov	al,0ffh			;underline
gflb2:	stosb
	pop	ds
	pop	dx
	pop	cx
	pop	si
	pop	bx
	ret

	
;Print byte word in AL to IR port in redeye format
redeye:
	push	bx			;save bx
	push	cx
	xor	ah,ah			;clear ah
	mov	bl,al			;store al
	and	al,78h
	call	findham
	jnc	bit2
	add	ah,8			;error bit 11=1
bit2:
	mov	al,bl
	and	al,0e6h
	call	findham
	jnc	bit3
	add	ah,4			;error bit 10=1
bit3:
	mov	al,bl
	and	al,0d5h
	call	findham
	jnc	bit4
	add	ah,2			;error bit 9=1
bit4:
	mov	al,bl		
	and	al,8bh
	call	findham
	jnc	dbit
	add	ah,1			;error bit 8 =1
dbit:		
	mov	al,bl			;restore al
	cli				;disable interrupts for timing
	call	one			;output start
	call	one	
	call	one
	mov	cl,12			;loop count
	mov	bx,800h			;mask bit
ploop:
	test	ax,bx			;check bit
	jnz	onebit			;bit is one
	call	zero			;output zero
	call	one
	jmp	elp
onebit:	call	one			;output one
	call	zero
elp:	
	shr	bx,1			;shift mask
	loop	ploop
	call	zero			;stop bit
	call	zero
	call	zero
	
	sti				;restore interrupts
	pop	cx
	pop	bx
	ret

findham:
	push	ax			;save ax
	push	cx			;save cx
	mov	cx,8
	xor	ah,ah			;zero ah
fhlp:
	shr	al,1
	jnc	efhlp
	xor	ah,0ffh		
efhlp:	loop	fhlp
	shr	ah,1			;set carry bit for return
	pop	cx
	pop	ax
	ret

;Output an 'on' halfbit in redeye format
one:	
	push	dx
	push	ax
	mov	dx,0e30ah		;irfmat
wtlp1:	in	al,dx	
	test	al,20h			;lbf
	jnz	wtlp1

	mov	al,2			;transmit on
	mov	dx,0e30bh		;ircnt
	out	dx,al
	pop	ax
	pop	dx
	ret

;Output an 'off' halfbit in redeye format
zero:	
	push	dx
	push	ax
	mov	dx,0e30ah		;irfmat
wtlp0:	in	al,dx	
	test	al,20h			;lbf
	jnz	wtlp0

	mov	al,0			;transmit off
	mov	dx,0e30bh		;ircnt
	out	dx,al
	pop	ax
	pop	dx
	ret

;short delay routine
sdelay:
	push	cx
	mov	cx,800
sdl1:	loop	sdl1
	pop	cx
	ret

exit:
	pop	si
	pop	di
	pop	dx
	pop	cx
	pop	bx
	pop	ax
	pop	es
	pop	ds
	iret

;==============================================================================

isr1c:
	pushf
	cli				; disable interrupts

	push	ds
	push	si
	push	ax			; save regs

	xor	si,si
	mov	ds,si			; point DS:SI at INT 5 vector
	mov	si,5*4

	cmp	word ptr [si],offset isr5
	jz	no_sysmgr_yet		; jif SysMgr hasn't taken it yet

	mov	word ptr [si],offset isr5
	mov	[si+2],cs		; else take it back for ourselves

	mov	si,4*1ch		; point DS:SI at INT 1C vector
	cmp	word ptr [si],offset isr1c
	jnz	no_sysmgr_yet		; jif we can't release INT 1C

	mov	ax,cs:old1coff		; restore old INT 1C vector
	mov	[si],ax
	mov	ax,cs:old1cseg
	mov	[si+2],ax
no_sysmgr_yet:
	pop	ax			; restore regs
	pop	si
	pop	ds
	popf				; restore interrupt flag
	jmp	dword ptr cs:old1coff

;==============================================================================
; INIT 
;==============================================================================

init:
	mov	ax,351ch
	int	21h		; get old INT 1C vector

	mov	old1coff,bx	; save it
	mov	old1cseg,es

	lea	dx,isr5		; take INT 5 vector
	mov	ax,2505h
	int	21h

	lea	dx,isr1c	; take INT 1C vector
	mov	ax,251ch
	int	21h

	lea	dx,copyr
	mov	ah,9		; print copyright message
	int	21h

	lea	dx,init		; TSR, through away INIT routine
	add	dx,15
	shr	dx,1
	shr	dx,1
	shr	dx,1
	shr	dx,1
	mov	ax,3100h
	int	21h

copyr	db	"Infrared Print Screen TSR rev 1.03",13,10
	db	"Copyright 1991 Hewlett-Packard Co.",13,10
	db	"All rights reserved",13,10,'$'

cseg	ends
	end	start
